<?php

namespace Drupal\tax_number\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'tax_number' field type.
 *
 * @FieldType(
 *   id = "tax_number",
 *   label = @Translation("Tax Number"),
 *   description = @Translation("Create and store tax identification numbers"),
 *   category = @Translation("Number"),
 *   default_widget = "tax_number_default",
 *   default_formatter = "tax_number_default",
 * )
 */
class TaxNumberItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Tax Number value'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
        'columns' => [
          'value' => [
            'type' => 'text',
            'size' => 'normal',
          ],
        ],
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('value')->getValue();
    return $value === NULL || $value === '';
  }

}
